@@ -13,7 +13,10 @@ quick_error! {
     pub enum Error {
         Io(err: io::Error) {
             display("Failed to read from line reader")
-            from()
+            source(err)
+        }
+        UploadPack(err: git_transport::packetline::read::Error) {
+            display("Upload pack reported an error")
             source(err)
         }
         Transport(err: client::Error) {
@@ -33,6 +36,22 @@ quick_error! {
     }
 }
 
+impl From<std::io::Error> for Error {
+    fn from(err: io::Error) -> Self {
+        if err.kind() == io::ErrorKind::Other {
+            match err.into_inner() {
+                Some(err) => match err.downcast::<git_transport::packetline::read::Error>() {
+                    Ok(err) => Error::UploadPack(*err),
+                    Err(err) => Error::Io(io::Error::new(io::ErrorKind::Other, err)),
+                },
+                None => Error::Io(io::ErrorKind::Other.into()),
+            }
+        } else {
+            Error::Io(err)
+        }
+    }
+}
+
 /// An 'ACK' line received from the server.
 #[derive(PartialEq, Eq, Debug, Hash, Ord, PartialOrd, Clone, Copy)]
 #[cfg_attr(feature = "serde1", derive(serde::Serialize, serde::Deserialize))]
